#include <qdockwindow.h>
#include <qpopupmenu.h>
#include <qstring.h>

#include <kaction.h>
#include <kapplication.h>
#include <kconfig.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>
#include <kmenubar.h>
#include <kstdaccel.h>
#include <ktoolbar.h>

#include "mainwindow.h"
#include "memcontroller.h"
#include "mempanel.h"
#include "memtheme.h"

#include <iostream>

enum GameSize { Small, Large, Huge };

//========================================
// @CLASS: MainWindow
//========================================
MainWindow::MainWindow(const char* name)
  : KMainWindow(0, name)
  , m_dock_window(0)
  , m_mem_panel(0)
  , m_mem_controller(0)
  , m_theme(0)
  , m_theme_name("default")
  , m_is_single_player(true)
  , m_game_size(Large)
  , m_n_match(2)
{
  readConfig();

  m_dock_window = new QDockWindow(QDockWindow::InDock, this, "Game status");
  addDockWindow(m_dock_window, "Game", Qt::DockRight);
  setDockWindowsMovable(false);

  m_theme = new MemTheme(QString("pics/")+m_theme_name);

  QPopupMenu* game_menu = new QPopupMenu(this);
  menuBar()->insertItem("&Game", game_menu);

  QPopupMenu* settings_menu = new QPopupMenu(this);
  menuBar()->insertItem("&Settings", settings_menu);

  KAction* openNew = KStdAction::openNew(this, SLOT(newGame()), actionCollection());
  openNew->plug(toolBar());
  openNew->plug(game_menu);

  toolBar()->insertSeparator();
  game_menu->insertSeparator();

  // ----- quit
  KAction* quit = KStdAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());
  quit->plug(game_menu);
  
  // ----- singlePlayer
  QIconSet singleIconSet(QPixmap("icons/single_player_16.png"),
			 QPixmap("icons/single_player_22.png"));
  KRadioAction* singlePlayer = new KRadioAction("&Single Player", singleIconSet,
						KShortcut(),
						this, SLOT(setSinglePlayer()),
						this);
  if (m_is_single_player) singlePlayer->setChecked(true);
  singlePlayer->setExclusiveGroup("NumPlayers");
  singlePlayer->plug(settings_menu);

  // ----- twoPlayers
  QIconSet twoIconSet(QPixmap("icons/two_players_16.png"),
		      QPixmap("icons/two_players_22.png"));
  KRadioAction* twoPlayers = new KRadioAction("&Two Players", twoIconSet,
					      KShortcut(),
					      this, SLOT(setTwoPlayers()),
					      this);
  if (!m_is_single_player) twoPlayers->setChecked(true);
  twoPlayers->setExclusiveGroup("NumPlayers");
  twoPlayers->plug(settings_menu);

  settings_menu->insertSeparator();

  // ----- smallGame
  KRadioAction* smallGame = new KRadioAction("S&mall (6x6)", QString::null, KShortcut(),
					     this, SLOT(setSmallGame()), this);
  if (m_game_size == Small) smallGame->setChecked(true);
  smallGame->setExclusiveGroup("GameSize");
  smallGame->plug(settings_menu);

  // ----- largeGame
  KRadioAction* largeGame = new KRadioAction("&Large (12x10)", QString::null, KShortcut(),
					     this, SLOT(setLargeGame()), this);
  if (m_game_size == Large) largeGame->setChecked(true);
  largeGame->setExclusiveGroup("GameSize");
  largeGame->plug(settings_menu);

  // ----- hugeGame
  KRadioAction* hugeGame = new KRadioAction("&Huge (16x12)", QString::null, KShortcut(),
					    this, SLOT(setHugeGame()), this);
  if (m_game_size == Huge) hugeGame->setChecked(true);
  hugeGame->setExclusiveGroup("GameSize");
  hugeGame->plug(settings_menu);

  settings_menu->insertSeparator();

  // ----- match2
  KRadioAction* match2 = new KRadioAction("Match &2", QString::null, KShortcut(),
					  this, SLOT(setMatch2()), this);
  if (m_n_match == 2) match2->setChecked(true);
  match2->setExclusiveGroup("NumMatch");
  match2->plug(settings_menu);

  // ----- match3
  KRadioAction* match3 = new KRadioAction("Match &3", QString::null, KShortcut(),
					  this, SLOT(setMatch3()), this);
  if (m_n_match == 3) match3->setChecked(true);
  match3->setExclusiveGroup("NumMatch");
  match3->plug(settings_menu);

  // ----- match4
  KRadioAction* match4 = new KRadioAction("Match &4", QString::null, KShortcut(),
					  this, SLOT(setMatch4()), this);
  if (m_n_match == 4) match4->setChecked(true);
  match4->setExclusiveGroup("NumMatch");
  match4->plug(settings_menu);

  settings_menu->insertSeparator();
  QPopupMenu* theme_menu = new QPopupMenu(this);
  settings_menu->insertItem("&Theme", theme_menu);

  // ----- defaultTheme
  KRadioAction* defaultTheme = new KRadioAction("Default", QString::null, KShortcut(),
						this, SLOT(setThemeDefault()), this);
  if (m_theme_name == "default") defaultTheme->setChecked(true);
  defaultTheme->setExclusiveGroup("Theme");
  defaultTheme->plug(theme_menu);

  // ----- slashdotTheme
  KRadioAction* slashdotTheme = new KRadioAction("Slashdot", QString::null, KShortcut(),
						 this, SLOT(setThemeSlashdot()), this);
  if (m_theme_name == "slashdot") slashdotTheme->setChecked(true);
  slashdotTheme->setExclusiveGroup("Theme");
  slashdotTheme->plug(theme_menu);

  menuBar()->insertItem("&Help", helpMenu(QString::null, false));

  newGame();
}

void MainWindow::newGame()
{
  if (m_mem_controller) {
    delete m_mem_controller;
    delete m_mem_panel;
  }

  int width, height;
  switch (m_game_size) {
  case Small:
    width = 6;
    height = 6;
    break;
  case Large:
    width = 12;
    height = 10;
    break;
  case Huge:
    width = 16;
    height = 12;
    break;
  }

  m_mem_panel = new MemPanel(this, m_theme, width, height, m_n_match);
  setCentralWidget(m_mem_panel);
  m_mem_panel->show();

  if (m_is_single_player) {
    m_mem_controller = new SinglePlayerMemController(m_dock_window, width*height, m_n_match);
  }
  else {
    m_mem_controller = new TwoPlayerMemController(m_dock_window, width*height, m_n_match);
  }
  m_dock_window->setWidget(m_mem_controller);
  std::cout << "Size hint: " << m_dock_window->sizeHint().width() 
            << ":" << m_dock_window->sizeHint().height() << std::endl;
  m_mem_controller->show();
  m_dock_window->setFixedExtentWidth(m_dock_window->sizeHint().width());

  setMinimumSize(sizeHint());
  setMaximumSize(sizeHint());

  QObject::connect(m_mem_panel, SIGNAL(move()), m_mem_controller, SLOT(move()));
  QObject::connect(m_mem_panel, SIGNAL(miss()), m_mem_controller, SLOT(miss()));
  QObject::connect(m_mem_panel, SIGNAL(guess(bool)), m_mem_controller, SLOT(guess(bool)));
  QObject::connect(m_mem_panel, SIGNAL(gameOver()), m_mem_controller, SLOT(gameOver()));
}

void MainWindow::setSinglePlayer()
{
  if (m_is_single_player != true) {
    m_is_single_player = true;
    settingsChanged();
  }
}

void MainWindow::setTwoPlayers()
{
  if (m_is_single_player != false) {
    m_is_single_player = false;
    settingsChanged();
  }
}

void MainWindow::setSmallGame()
{
  if (m_game_size != Small) {
    m_game_size = Small;
    settingsChanged();
  }
}

void MainWindow::setLargeGame()
{
  if (m_game_size != Large) {
    m_game_size = Large;
    settingsChanged();
  }
}

void MainWindow::setHugeGame()
{
  if (m_game_size != Huge) {
    m_game_size = Huge;
    settingsChanged();
  }
}

void MainWindow::setMatch2()
{
  if (m_n_match != 2) {
    m_n_match = 2;
    settingsChanged();
  }
}

void MainWindow::setMatch3()
{
  if (m_n_match != 3) {
    m_n_match = 3;
    settingsChanged();
  }
}

void MainWindow::setMatch4()
{
  if (m_n_match != 4) {
    m_n_match = 4;
    settingsChanged();
  }
}

void MainWindow::setThemeDefault()
{
  if (m_theme_name != "default") {
    m_theme_name = "default";
    delete m_theme;
    m_theme = new MemTheme("pics/default");
  }
}

void MainWindow::setThemeSlashdot()
{
  if (m_theme_name != "slashdot") {
    m_theme_name = "slashdot";
    delete m_theme;
    m_theme = new MemTheme("pics/slashdot");
  }
}

void MainWindow::settingsChanged()
{
  writeConfig();
  if (m_mem_controller->is_started()) {
    int answer = KMessageBox::questionYesNo(0,
                                            i18n("Abandon current game?\n\n"
                                                 "If you choose 'No', the new settings "
                                                 "will be applied next time you start a new game."));
    if (answer == KMessageBox::Yes) {
      newGame();
    }
  }
  else {
    newGame();
  }
}

void MainWindow::readConfig()
{
  KConfig* config = KApplication::kApplication()->config();
  config->setGroup("");

  m_is_single_player = config->readBoolEntry("Single Player", m_is_single_player);
  m_game_size = config->readNumEntry("Game Size", m_game_size);
  m_n_match = config->readNumEntry("Match Size", m_n_match);
  m_theme_name = config->readEntry("Theme", m_theme_name);
}

void MainWindow::writeConfig()
{
  KConfig* config = KApplication::kApplication()->config();
  config->setGroup("");

  config->writeEntry("Single Player", m_is_single_player);
  config->writeEntry("Game Size", m_game_size);
  config->writeEntry("Match Size", m_n_match);
  config->writeEntry("Theme", m_theme_name);
  config->sync();
}
