// -*- mode: c++; -*-

#include <vector>
#include <qwidget.h>

class QCheckBox;
class QLCDNumber;
class QVBoxLayout;
class QTimer;

//========================================
// @CLASS: MemController
//========================================
class MemController : public QWidget {
  Q_OBJECT
public:
  MemController(QWidget* parent, int n_match);
  virtual ~MemController();

  virtual bool is_started() const = 0;

public slots:
  virtual void move() = 0;
  virtual void miss() = 0;
  virtual void guess(bool firstSeen) = 0;
  virtual void gameOver() = 0;

private slots:
  virtual void tick();

protected:
  void init();

protected:
  QVBoxLayout* m_layout;

private:
  void highlightMatched();

protected:
  int m_minutes, m_seconds;
  int m_n_match;

private:
  QLCDNumber* m_time_display;
  QTimer* m_timer;
  std::vector<QCheckBox*> m_match_status;
  int m_matched;
};

//========================================
// @CLASS: SinglePlayerMemController
//========================================
class SinglePlayerMemController : public MemController {
public:
  SinglePlayerMemController(QWidget* parent, int num_items, int n_match);

  virtual bool is_started() const;

public:
  virtual void move();
  virtual void miss();
  virtual void guess(bool firstSeen);
  virtual void gameOver();

private:
  int m_num_items;
  int m_num_moves;
  int m_score;
  int m_matched_items;
  QLCDNumber* m_moves_display;
  QLCDNumber* m_score_display;
};

//========================================
// @CLASS: TwoPlayerMemController
//========================================
class TwoPlayerMemController : public MemController {
public:
  TwoPlayerMemController(QWidget* parent, int num_items, int n_match);

  virtual bool is_started() const;
public:
  virtual void move();
  virtual void miss();
  virtual void guess(bool firstSeen);
  virtual void gameOver();

private:
  void highlight_turn();

private:
  int m_turn;
  int m_num_items;
  int m_num_moves_1;
  int m_num_moves_2;
  int m_score_1;
  int m_score_2;
  int m_matched_items;
  QLCDNumber* m_score_1_display;
  QLCDNumber* m_score_2_display;
  QLCDNumber* m_moves_1_display;
  QLCDNumber* m_moves_2_display;
};

