#include <qcanvas.h>
#include <qdir.h>
#include <qevent.h>
#include <qpainter.h>
#include <qpoint.h>

#include <iostream>
#include <time.h>
#include <unistd.h>

#include "mempanel.h"
#include "memtheme.h"

class MemPanel::CardInfo {
public:
  CardInfo(int index, QCanvasSprite* sprite)
    : m_index(index)
    , m_times_seen(0)
    , m_sprite(sprite)
  {
  }
  ~CardInfo() {
    delete m_sprite;
  }

  int m_index;
  int m_times_seen;
  QCanvasSprite* m_sprite;
};

MemPanel::MemPanel(QWidget* parent, MemTheme* theme,
		   int num_cols, int num_rows, int num_match)
  : QCanvasView(parent, "MemPanel")
  , m_num_rows(num_rows)
  , m_num_cols(num_cols)
  , m_num_match(num_match)
  , m_theme(theme)
{
  setCanvas(new QCanvas(num_cols*60,num_rows*60));
  setResizePolicy(QScrollView::AutoOne);
  setMinimumSize(sizeHint());
  setMaximumSize(sizeHint());

  if (parent) {
    QPixmap* pixmap = parent->backgroundBrush().pixmap();
    if (pixmap) {
      canvas()->setBackgroundPixmap(*pixmap);
      setBackgroundPixmap(*pixmap);
      setBackgroundMode(Qt::FixedPixmap);
    }
  }

  init();
  canvas()->update();
}

MemPanel::~MemPanel()
{
  for (int i=0; i<m_cards.size(); ++i) {
    delete m_cards[i];
  }
}

void MemPanel::init()
{
  QPixmap card_cover(50,50);
  QPainter painter;
  painter.begin(&card_cover);
  painter.setPen(Qt::black);
  painter.setBrush(Qt::gray);
  painter.drawRect(0,0,50,50);
  painter.end();
  QCanvasPixmap cover_pix(card_cover, QPoint(0,0));

  int x=0, y=0;
  int index = 0;
  m_theme->reset();
  for (; ; ++index) {
    const QPixmap& card_contents = m_theme->getRandom();

    for (int group_i=0; group_i < m_num_match; ++group_i)
    {
      QCanvasPixmapArray* pix_arr = new QCanvasPixmapArray();
      pix_arr->setImage(0, new QCanvasPixmap(card_cover, QPoint(0,0)));
      pix_arr->setImage(1, new QCanvasPixmap(card_contents, QPoint(0,0)));
      
      QCanvasSprite* sprite = new QCanvasSprite(pix_arr, canvas());
      sprite->move(x*60+5,y*60+5);
      sprite->show();
      
      m_cards.push_back(new CardInfo(index, sprite));
      
      x++;
      if (x==m_num_cols) {
	x=0;
	y++;
      }
    }

    if (y == m_num_rows) break;
  }
  
  srand(time(0)+getpid());
  for (int i=0; i<m_num_rows*m_num_cols*5; i++) {
    QCanvasSprite* s1 = m_cards[rand()%(m_num_cols*m_num_rows)]->m_sprite;
    QCanvasSprite* s2 = m_cards[rand()%(m_num_cols*m_num_rows)]->m_sprite;
    double tmp1,tmp2;
    tmp1 = s1->x();
    tmp2 = s2->x();
    s1->setX(tmp2);
    s2->setX(tmp1);

    tmp1 = s1->y();
    tmp2 = s2->y();
    s1->setY(tmp2);
    s2->setY(tmp1);
  }
}

int MemPanel::findCard(QMouseEvent* e)
{
  int found_card = -1;

  QCanvasItemList l = canvas()->collisions(e->pos());
  for (QCanvasItemList::Iterator ci=l.begin(); ci != l.end(); ++ci) {
    for (int i=0; i<m_cards.size(); ++i) {
      if (m_cards[i]->m_sprite == *ci) {
	found_card = i;
	break;
      }
    }
  }

  return found_card;
}

void MemPanel::contentsMousePressEvent(QMouseEvent* e)
{
  int found_card = findCard(e);
  if (found_card == -1) return;

  if (m_cards[found_card]->m_sprite->frame() == 1) {
    return;
  }
  if (m_selected_cards.size()==m_num_match) {
    return;
  }
  m_cards[found_card]->m_times_seen++;
  m_cards[found_card]->m_sprite->setFrame(1);
  m_selected_cards.push_back(found_card);

  emit move();
  
  canvas()->update();
}

void MemPanel::contentsMouseReleaseEvent(QMouseEvent* e)
{
  if (m_selected_cards.size() == m_num_match) {
    bool missed = false;
    bool first_seen = true;

    int first_index = m_cards[m_selected_cards[0]]->m_index;
    for (int i=0; i<m_num_match; ++i) {
      CardInfo* ci = m_cards[m_selected_cards[i]];
      if (ci->m_times_seen > 2) {
	first_seen = false;
      }
      if (ci->m_index != first_index) {
	missed = true;
	emit miss();

	for (int i=0; i<m_num_match; ++i) {
	  m_cards[m_selected_cards[i]]->m_sprite->setFrame(0);
	}
	m_selected_cards.resize(0);
	canvas()->update();
	return;
      }
    }

    m_selected_cards.resize(0);
    emit guess(first_seen);
    std::cout << "Guessed" << std::endl;
  }
  canvas()->update();
}

