#include <assert.h>
#include <algorithm>
#include <time.h>
#include <unistd.h>

#include <qdir.h>
#include <qpixmap.h>
#include <qstringlist.h>

#include "memtheme.h"

MemTheme::MemTheme(const char* path)
  : m_used(0)
{
  QDir dir(path);
  dir.setFilter(QDir::Files|QDir::Readable);
  QStringList pixmaps(dir.entryList());

  for (QStringList::Iterator i=pixmaps.begin(); i != pixmaps.end(); ++i) {
    QPixmap* pixmap = new QPixmap(dir.filePath(*i));
    m_images.push_back(pixmap);
  }
}

MemTheme::~MemTheme()
{
  for (int i=0; i<m_images.size(); ++i) {
    delete m_images[i];
    m_images[i] = 0;
  }
}

void MemTheme::reset()
{
  srand(time(0)+getpid());
  for (int i=0; i<m_images.size(); ++i) {
    std::swap(m_images[rand()%m_images.size()], m_images[rand()%m_images.size()]);
  }
  m_used = 0;
}

const QPixmap& MemTheme::getRandom()
{
  assert(m_used < m_images.size());
  return *m_images[m_used++];
}
